/** \file alignment.cpp
 * \brief \ref alignment class methods implementation. 
 *
 * In this file the \ref alignment class methods are implemented.
 */

#include "alignment.h"

#define DELIMITERS " \t\n"


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| alignment::alignment() 				|
|  	Class constructor                               |
|  							|
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment::alignment(void) {
  
  /* Init variable value to 0 */
  species = 0;
  aminos =  0;
  
  /* And pointer value to NULL */
  alignmentMatrix = NULL;
  speciesNames =    NULL;
  sgaps = 	    NULL;
  scons =           NULL;
  seqMatrix = 	    NULL;

}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| alignment::alignment( int _species, ..., ... ) 	|
|  	Class constructor                               |
|  							|
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment::alignment(char **Matrix, char **Names, int _species, int _aminos) {

  int i, length;
	
  /* Assign the parameter values to the variables */
  species = _species;
  aminos =  _aminos;
  
  speciesNames = new char*[species];
  for(i = 0; i < species; i++) {		
    length = strlen(Names[i]);
    speciesNames[i] = new char[length + 1];    
    strcpy(speciesNames[i], Names[i]);
  }
  
  alignmentMatrix = new char*[species];  
  for(i = 0; i < species; i++) {
    alignmentMatrix[i] = new char[aminos + 1];
    strcpy(alignmentMatrix[i], Matrix[i]);
  }
  
  sgaps =     NULL;
  scons =     NULL;
  seqMatrix = NULL;
  
}

  
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| alignment::~alignment()			 	|
|  	Class destructor				|
|  							|
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
  
alignment::~alignment(void) {

  if(alignmentMatrix != NULL)
    freeAlignment();
  
}

  
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::freeAlignment() 	                |
|  	Attribute memory deletion                       |
|  							|
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
  
void alignment::freeAlignment(void) {
  int i;  
  
  for(i = 0; i < species; i++)
    delete[] alignmentMatrix[i];
  delete[] alignmentMatrix;

  for(i = 0; i < species; i++)
    delete[] speciesNames[i];
  delete[] speciesNames;

  if(sgaps != NULL)
    delete sgaps;
  
  if(scons != NULL)
    delete scons;
  
  species = 0;
  aminos =  0;
  
  alignmentMatrix = NULL;
  speciesNames = NULL;
  sgaps = NULL;
  scons = NULL;

}  

  
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
|  bool alignment::loadAlignment(char *alignmentFile)	|
|  	This method loads an alignment from a file. The |
|       file is in PHYLIP2 format                       |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
bool alignment::loadAlignment(char *alignmentFile) {
  
  int i, j, length = 0;
  char line[256], *str;
  ifstream file;
  
  /* Open alignment file and open check */
  file.open(alignmentFile, ifstream::in);
  if(!file) return false;
  if(alignmentMatrix != NULL)
    freeAlignment();
  
  /* Read the number of species and the number of aminos from the first line */
  file.getline(line, 256);

  /* Check the file first line */
  if(!strcmp(line, "")) 
    return false;

  species = atoi(strtok(line, DELIMITERS));
  aminos  = atoi(strtok(NULL, DELIMITERS));

  /* Check correct read parameters */
  if((species == 0) || (aminos == 0)) 
    return false;

  /* Allocate memory for alignment matrix and species names vector */
  alignmentMatrix = new char*[species];
  speciesNames =    new char*[species];

  for(i = 0; i < species; i++)
    alignmentMatrix[i] = new char[aminos+1];
  
  /* Read the first block of lines that contains two tokens */
  for(i = 0; i < species; i++){
    file.getline(line, 256);
    
    /* First token: species name */
    str = strtok(line, DELIMITERS);
    length = strlen(str);
    speciesNames[i] = new char[length+1];
    strcpy(speciesNames[i], str);
    
    /* Next token: first N aminoacid block */
    str = strtok(NULL, DELIMITERS);
    strcpy(alignmentMatrix[i], str);   

    /* We calculate the line length. */
    length = strlen(str);

    /* we clean the line of spaces and tabulations */
    /* and store it in the alignment matrix        */    
    while(1) {
      str = strtok(NULL, DELIMITERS);
      if(str != NULL) {
        length += strlen(str);
        strcat(alignmentMatrix[i], str);
      }
      else break;
    }
  }
  
  /* Read the next aminoacid blocks */
  for(i = length; i < aminos; i += length) {
    
    /* we clean any number of blank lines between blocks */
    do {
       file.getline(line, 256);
    } while(!strcmp(line, ""));

    /* we clean the line of spaces and tabulations */
    /* and store it in the alignment matrix        */
    str = strtok(line, DELIMITERS);  
    strcat(alignmentMatrix[0], str);

    while(1) {
      str = strtok(NULL, DELIMITERS);
      if(str != NULL) strcat(alignmentMatrix[0], str);
      else break;
    }
 
    /* For the anothers lines, we clean the line of spaces   */
    /* and tabulations and store it in the alignment matrix  */
    for(j = 1; j < species; j++) {

      file.getline(line, 256);
      str = strtok(line, DELIMITERS);  
      strcat(alignmentMatrix[j], str);

      while(1) {
        str = strtok(NULL, DELIMITERS);
        if(str != NULL) strcat(alignmentMatrix[j], str);
        else break;
      }
    }
  }

  /* Close the alignment file */
  file.close();
  
  /* All was OK, return true */
  return true;
  
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::alignmentToFile(ostream &file)	|
|  	Private method that put the alignment on the    |
|  	parameter stream in PHYLIP2 format              |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::alignmentToFile(ostream &file) {
  
  int i, j, maxLongName = 0;
  char str[61];
  
  /* Include in the first line the species and the aminoacids of the alignment */
  file << species << " " << aminos << endl;
  
  /* Look for the maximum species name size */
  for(i = 0; i < species; i++)
    maxLongName = utils::max(maxLongName, strlen(speciesNames[i])); 
  
  /* Concatenate '\0' to the final of the str string */
  str[60] = '\0';
  
  /* Put on the stream the species names and the first 60 aminoacids block */
  for(i = 0; i < species; i++) {
    file << setw(maxLongName+2) << left << speciesNames[i];
    strncpy(str, alignmentMatrix[i], 60);
    file << str << endl;
  }
  file << endl;
  
  /* Put on the stream the rest of the blocks */
  for(i = 60; i < aminos; i += 60) {
    for(j = 0; j < species; j++) {
      strncpy(str, &alignmentMatrix[j][i], 60);
      file << str << endl;
    }
    file << endl;
  }
  
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| bool alignment::printAlignment()			|
|  	This method prints the alignment to the standard|
|	output in PHYLIP2 format                        |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

bool alignment::printAlignment(void){
  
  if(alignmentMatrix == NULL)
    return false;
  
  alignmentToFile(cout);
  return true;
  
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| bool alignment::saveAlignment(char *destFile)		|
|  	This method puts the alignment on the file      |
|  	in PHYLIP2 format				|
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

bool alignment::saveAlignment(char *destFile) {
  
  ofstream file;
  
  /* File open and correct open check */
  file.open(destFile);
  if(!file || alignmentMatrix == NULL) 
    return false;

  /* Alignment saving */
  alignmentToFile(file);

  /* Destination file closing */
  file.close();
  
  /* All was OK, return true */
  return true;
  
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| alignment *alignment::cleanGaps(float baseLine, float threshold) |
|       This method cleans the alignment using the two input       |
|       parameters and using gaps method. This parameters are:     |
|                                                                  |
|       - baseline:  % of columns of the alignment that  we        |
|       always are conservating, security minimum                  |
|       - threshold: % of gaps per column that puts the            |
|       inferior limit that every column of the clean alignment    |
|       will have                                                  |
|                                                                  |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
		
alignment *alignment::cleanGaps(float baseLine, float threshold) {
  
  alignment *ret;
  int cut;
  
  /* If gap statistics are not calculated, we calculate them */
  if(!calculateGapStats())
    return NULL;
  
  /* Obtain the cut point using the given parameters */
  cut = sgaps -> calcCutPoint(baseLine, threshold);
    
  /* With the cut value and the gapsWindow vector of gap statistics  */
  /* we clean the alignment and generate a new alignment object with */  
  /* the clean alignment                                             */
  ret = cleanByCutValue(cut, sgaps -> getGapsWindow());
  
  /* Return a reference of the clean alignment */
  return ret;
  
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| alignment *alignment::cleanConservation(float baseLine, float conservationPct) |
|  	This method cleans the alignment using the two input                     |
|	parameters and the conservation method . This parameters are:            |
|	                                                                         |
|       - baseline:  % of columns of the alignment that  we                      |
|	always are conservating, security minimum                                |
|  	- conservationPct: minimum conservation value (0-1) permitted            |
|       for the new clean alignment                                              |
|	                                                                         |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment *alignment::cleanConservation(float baseLine, float conservationPct){

  /* With this we check that conservation vectors are calculated and */
  /* we can calculate a cut point                                    */
  if(scons == NULL) return NULL;

  if(!scons->vectorsCalculated()) return NULL;
  
  /* Calculate the cut point using the given parameters */
  float value = scons->calcCutPoint(baseLine, conservationPct);
  
  /* Clean the alignment and generate a new alignment object */
  /* that contains the clean alignment                       */
  alignment *ret = cleanByCutValue(value, scons->getMdkwVector());
  
  /* Return a refernce of the new alignment */
  return ret;
  
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| alignment * alignment::clean(float baseLine, float threshold, float conservationPct) |
|  	This method cleans the alignment using three input                             |
|	parameters and both gaps method and conservation method . This parameters are: |
|	                                                                               |
|       - baseline:  % of columns of the alignment that  we                            |
|	always are conservating, security minimum                                      |
|  	- conservationPct: minimum conservation value (0-1) permitted                  |
|       for the new clean alignment                                                    |
|  	- threshold: % of gaps per column that sets the                                |
|	inferior limit that every column of the clean alignment                        |
|	will have                                                                      |
|	                                                                               |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment *alignment::clean(float baseLine, float threshold, float conservationPct){
  
  int cutGaps;
  float cutCons;
  
  /* With this we check that conservation vectors are calculated and */
  /* we can calculate a cut point                                    */
  if(scons == NULL || sgaps == NULL)
    return NULL;
  
  if(!scons->vectorsCalculated())
    return NULL;
  
  /* Calculate the two cut points using the given parameters */
  /* A cut point used for gap cleaning, and a cut point used */
  /* for conservation cleaning                               */
  cutGaps = sgaps->calcCutPoint(baseLine, threshold);
  cutCons = scons->calcCutPoint(baseLine, conservationPct);
  
  /* Clean the alingment using the two cut values and the gapsWindow vector */
  /* and the mdkw values vector                                             */
  alignment *ret = cleanByCutValue(cutGaps, sgaps->getGapsWindow(), cutCons, scons->getMdkwVector());
  
  /* Return a reference of the clean alignment object */
  return ret;
  
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| alignment *alignment::cleanCompareFile(float threshold, float *vectValues)     |
|  	This method cleans the alignment using the two input                     |
|	parameters:                                                              |
|	                                                                         |
|       - threshold: Lower limit (0-1) of comparefile value admits in the        |
|                    alignment.                                                  |
|  	- vectValues: A vector with alignment's comparefile values.              |
|	                                                                         |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment *alignment::cleanCompareFile(float threshold, float *vectValues) {

  /* Clean the alignment and generate a new alignment object */
  /* that contains the clean alignment                       */
  alignment *ret = cleanByCutValue(threshold, vectValues);
  
  /* Return a refernce of the new alignment */
  return ret;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
|  alignment *alignment::cleanMixSlope(void)					       |
|	This method cleans, automaticly, the alignment file using the combined method  |
|	between the first and second slope of ratio between alignment's length and gaps|
|	percentage.  								       |
|										       |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment *alignment::cleanMixSlope(void) {

  int cut; alignment *ret;

  if(sgaps == NULL)
    sgaps = new statisticsGaps(alignmentMatrix, species, aminos);
  
  cut = sgaps -> calcCutPointMixSlope(); 
  ret = cleanByCutValue(cut, sgaps->getGapsWindow());

  return ret;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
|  alignment *alignment::cleanMixSlope(void)					       |
|	This method cleans, automaticly, the alignment file using the second slope of  |
|       ratio between alignment's length and gaps percentage.                          |
|										       |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment *alignment::clean2ndSlope(void) {

  int cut; alignment *ret;

  if(sgaps == NULL)
    sgaps = new statisticsGaps(alignmentMatrix, species, aminos);
  
  cut = sgaps -> calcCutPoint2ndSlope(); 
  ret = cleanByCutValue(cut, sgaps->getGapsWindow());

  return ret;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
|  alignment *alignment::cleanByCutValue(int, const int *)      |
|	This method uses a given cut point value and a          |
|  	gapswindow vector to clean the columns in the alignment |
|  	of which value is less than the cut point value         |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
		
alignment *alignment::cleanByCutValue(int cut, const int *gInCol) {
  
  int i, j, k, newAminos = 0;
  char **matrixAux;
  alignment *newAlig;
  
  /* Calculate the new matrix number of columns and allocate memory for the new matrix */
  for(i = 0; i < aminos; i++)
    if(gInCol[i] <= cut) 
      newAminos++;		
  
  matrixAux = new char*[species];
  for(i = 0; i < species; i++)
    matrixAux[i] = new char[newAminos+1];
  
  /* Copy the columns from the alignment to the new alignment, only if  */
  /* gapswindow value of the column is less than the cut value          */
  for(i = 0, k = 0; i < aminos; i++)
    if(gInCol[i] <= cut) {   // GAPS CUT FILTERING
      for(j = 0; j < species; j++)
	matrixAux[j][k] = alignmentMatrix[j][i];
      
      k++;
    }

  /* Concatenate the character '\0' to the last position of all rows of the alignment */
  /* to facilitate character vector manipulation and printing function                */  
  for(i = 0; i < species; i++)
    matrixAux[i][newAminos] = '\0';
  
  /* When we have all parameters, we create the new alignment */
  newAlig = new alignment(matrixAux, speciesNames, species, newAminos);
  
  /* Allocated auxiliar memory deletion */
  for(i = 0; i < species; i++)
    delete[] matrixAux[i];
  
  delete[] matrixAux;
  
  /* Return the new alignment reference */
  return newAlig;
  
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
|  alignment *alignment::cleanByCutValue(float, const float *) |
|	This method uses a given cut point value and a         |
|  	mdkwindow vector to clean the columns in the alignment |
|  	of which value is greater than the cut point value     |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment *alignment::cleanByCutValue(float cut, const float *vector) {

  int i, j, k, newAminos = 0;
  char **matrixAux;
  alignment *newAlig;
 		
  /* Calculate the new matrix number of columns and allocate memory for the new matrix */
  for(i = 0; i < aminos; i++)
    if(vector[i] >= cut) 
      newAminos++;	
  
  matrixAux = new char*[species];
  for(i = 0; i < species; i++)
    matrixAux[i] = new char[newAminos+1];
  
  /* Copy the columns from the alignment to the new alignment, only if */
  /* mdkwindow value of the column is greater than the cut value       */
  for(i = 0, k = 0; i < aminos; i++)
    if(vector[i] >= cut) {   // CONSERVATION CUT FILTERING
      for(j = 0; j < species; j++)
	matrixAux[j][k] = alignmentMatrix[j][i];
      
      k++;
    }

  /* Concatenate the character '\0' to the last position of all rows of the alignment */
  /* to facilitate character vector manipulation and printing function                */  
  for(i = 0; i < species; i++)
    matrixAux[i][newAminos] = '\0';
  
  /* When we have all parameters, we create the new alignment */
  newAlig = new alignment(matrixAux, speciesNames, species, newAminos);
  
  /* Allocated auxiliar memory deletion */
  for(i = 0; i < species; i++)
    delete[] matrixAux[i];
  
  delete[] matrixAux;
  
  /* Return the new alignment reference */
  return newAlig;
  
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
|  alignment *alignment::cleanByCutValue(int, const int *, float, const float *)  |
|	This method uses two given cut point values: gap method cut point and     |
|  	conservation method cut point. This method cleans the colums of which     |
|  	gapswindow value is less than gap method cut point and of which mdkwindow |
|       value is greater than conservation method cut point.                      |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

alignment *alignment::cleanByCutValue(int cutGaps, const int *gInCol, float cutCons, const float *MDK_W){

  int i, j, k, newAminos = 0;
  char **matrixAux;
  alignment *newAlig;


  /* Calculate the new matrix number of columns and allocate memory for the new matrix */
  for(i = 0; i < aminos; i++)
    if(MDK_W[i] >= cutCons && gInCol[i] <= cutGaps) // Here we filter using two different values and two different vectors
      newAminos++;
  
  matrixAux = new char*[species];
  for(i = 0; i < species; i++)
    matrixAux[i] = new char[newAminos+1];
  
  /* Copy the columns from the alignment to the new alignment, only if */
  /* mdkwindow value of the column is greater than the cut value       */
  for(i = 0, k = 0; i < aminos; i++)
    if(MDK_W[i] >= cutCons && gInCol[i] <= cutGaps) {	// GAPS CUT AND CONSERVATION CUT FILTERING
      for(j = 0; j < species; j++)
	matrixAux[j][k] = alignmentMatrix[j][i];
      
      k++;
    }
  
  /* Concatenate the character '\0' to the last position of all rows of the alignment */
  /* to facilitate character vector manipulation and printing function                */  
  for(i = 0; i < species; i++)
    matrixAux[i][newAminos] = '\0';
  
  /* When we have all parameters, we create the new alignment */
  newAlig = new alignment(matrixAux, speciesNames, species, newAminos);
  
  /* Allocated auxiliar memory deletion */
  for(i = 0; i < species; i++)
    delete[] matrixAux[i];
  
  delete[] matrixAux;
  
  /* Return the new alignment reference */
  return newAlig;

}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| bool alignment::calculateConservationStats()	            |
|  	Method that calculates conservation stats only if a |
|  	similarity matrix was defined in the alignment      |
|       with a previous call to                             |
|       calculateConservationStats(similarityMatrix)        |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

bool alignment::calculateConservationStats(void) {

  /* In this method we recalculate conservation statistics using a       */
  /* previously "installed" similarity matrix. If there is no previously */
  /* installed similarity matrix the method returs false.                */
  /* If the scons object is not created, we also return false            */

  if(scons != NULL)
    if(scons->isSimMatrixDef()){

      /* Here we don't check if calculateVectors() method and applyWindow()     */
      /* method returns true or false, because if object exists and this object */
      /* has a valid similarity matrix, all is going to be ok, instead of       */
      /* applyWindow() method returns false (we use the same windows size)      */          

      scons->calculateVectors(alignmentMatrix, species, aminos);
      scons->applyWindow();
      return true;
    }
  
  return false;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| bool alignment::calculateConservationStats()	            |
|  	Method that calculates conservation stats using the |
|       similarity matrix given                             |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

bool alignment::calculateConservationStats(similarityMatrix *sm){

  /* If gap statistics are not calculated, we calculate them */
  if(!calculateGapStats())
    return false;
  
  /* If scons object is not created, we create them */
  if(scons == NULL)
    scons = new statisticsConservation(alignmentMatrix, species, aminos, sgaps);
  
  /* If the similarity matrix is wrong, return false */
  if(!scons->setSimilarityMatrix(sm))
    return false;
  
  if(!scons->calculateVectors(alignmentMatrix, species, aminos))
    return false;
  
  /* Apply window */
  bool ret = scons->applyWindow();

  return ret;

}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| bool alignment::calculateGapStats()	                |
|       Method that calculates gap statistics if        |
|       the statistics haven't been calculated yet      |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

bool alignment::calculateGapStats(void){
  
  if(alignmentMatrix == NULL) 
    return false;
  
  if(sgaps == NULL)
    sgaps = new statisticsGaps(alignmentMatrix, species, aminos);
  
  return true;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::calculateGapStats(hwindow)            |
|       Method that calculates gap statistics if        |
|       the statistics haven't been calculated yet, and |
|       calculates window statistics using the half     |
|       window size given                               |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

bool alignment::calculateGapStats(int hwindow){
  
  if(!calculateGapStats())
    return false;

  /* Apply window */
  bool ret = sgaps->applyWindow(hwindow);

  return ret;
  
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::printStatisticsGapsColumns()          |
|       Method that prints the number of gaps of each   |
|       column of the alignment (with previous window   |
|       calculation)                                    |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::printStatisticsGapsColumns(void) {

  if(sgaps == NULL)
    sgaps = new statisticsGaps(alignmentMatrix, species, aminos);
  
  sgaps -> printGapsColumns();
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::printStatisticsGapsTotal()             |
|       Method that prints the accumulated number of     |
|       columns for each number of gaps in the alignment |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::printStatisticsGapsTotal(void) {

  if(sgaps == NULL)
    sgaps = new statisticsGaps(alignmentMatrix, species, aminos);
  
  sgaps -> printGapsAcl();
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::printStatisticsConservationColumns()   |
|       Method that prints the conservation value of     |
|       each column of the alignment (with previous      |
|       statisticsConservation calculation)              |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::printStatisticsConservationColumns(void) {

  if(sgaps == NULL)
    sgaps = new statisticsGaps(alignmentMatrix, species, aminos);

  /* If scons object is not created, we create them */
  if(scons == NULL) {
    scons = new statisticsConservation(alignmentMatrix, species, aminos, sgaps);

    if(scons -> isSimMatrixDef()) {
      scons -> calculateVectors(alignmentMatrix, species, aminos);
      scons -> applyWindow();
    }
  }
  scons -> printConservationColumns(aminos);
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::printStatisticsConservationTotal()     |
|       Method that prints the accumulated number of     |
|       columns for each conservation value from the     |
|       the alignment (with previous statistics          |
|       Conservation calculation)                        |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::printStatisticsConservationTotal(void) {

  if(sgaps == NULL)
    sgaps = new statisticsGaps(alignmentMatrix, species, aminos);

  /* If scons object is not created, we create them */
  if(scons == NULL) {
    scons = new statisticsConservation(alignmentMatrix, species, aminos, sgaps);

    if(scons -> isSimMatrixDef()) {
      scons -> calculateVectors(alignmentMatrix, species, aminos);
      scons -> applyWindow();
    }
  }
  scons -> printConservationAcl(aminos);
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| int alignment::getNumSpecies(void)                       |
|       Method that returns the alignment's species number |
|                                                          |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

int alignment::getNumSpecies(void) {

  return species;

}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::getSpecies(char **Names)               |
|       Method that returns the alignment's species name |
|                                                        |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::getSpecies(char **Names) {

  for(int i = 0; i < species; i++)
    strcpy(Names[i], speciesNames[i]);

}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| int alignment::getNumAminos(void)                             |
|       Method that returns the alignment's amino acids number  |
|                                                               |
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

int alignment::getNumAminos(void) {

  return aminos;

}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::sequenMatrix(void)                       |
|       Method that builds an alignment's sequences matrix |
|                                                          |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::sequenMatrix(void) {

  if(seqMatrix == NULL)
    seqMatrix = new sequencesMatrix(alignmentMatrix, species, aminos);

}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::printSequenMatrix(void)                |
|       Method that prints the alignment's sequence      |
|       matrix                                           |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::printSequenMatrix(void) {

  if(seqMatrix == NULL)
    seqMatrix = new sequencesMatrix(alignmentMatrix, species, aminos);
  
  seqMatrix -> printMatrix();
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::getColumSeqMatrix(int, int *)          |
|       Method that gets a "column"-column from the      |
|       alignment's sequence matrix                      |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::getColumnSeqMatrix(int column, int *columnSeqMatrix) {

  if(seqMatrix == NULL)
    seqMatrix = new sequencesMatrix(alignmentMatrix, species, aminos);

  seqMatrix -> getColumn(column, columnSeqMatrix);
	
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++
| void alignment::getColumSeqMatrix(int, int, int *)     |
|       Method that gets a column from the aligment's    |
|       sequence matrix with the same value that "value" |
|       at matrix's position (row, i)                    |
++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void alignment::getColumnSeqMatrix(int value, int row, int *columnSeqMatrix) {

  if(seqMatrix == NULL)
    seqMatrix = new sequencesMatrix(alignmentMatrix, species, aminos);

  seqMatrix -> getColumn(value, row, columnSeqMatrix);

}

