/** \file alignment.h
 * \brief File containing \ref alignment class definition.
 *
 * This file contains the \ref alignment class definition, with the
 * definition of its attributes and its methods.
 */

#ifndef ALIGNMENT_H
#define ALIGNMENT_H

#include <fstream.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>
#include <iomanip.h>

#include "statisticsGaps.h"
#include "sequencesMatrix.h"
#include "statisticsConservation.h"
#include "similarityMatrix.h"
#include "utils.h"


/** \brief Class containing an alignment
 *
 * This class stores the alignment. It provides methods
 * to \b clean the alignment and generate the clean alignment.
 * It also provides methods for \b statistics \b calculation and
 * \b statistics \b printing.
 */

class alignment {
  int species;
  int aminos;
  char **alignmentMatrix;
  char **speciesNames;

  /* Sequences */
  sequencesMatrix *seqMatrix;

  /* Statistics */
  statisticsGaps *sgaps;  
  statisticsConservation *scons;
  
 private: 
  /* Alignment to a stream */	
  void alignmentToFile(ostream &file);
  
  /* Alignment cleaning */
  alignment *cleanByCutValue(int cut, const int *gInCol);
  alignment *cleanByCutValue(float cut, const float *vector);
  alignment *cleanByCutValue(int cutGaps, const int *gInCol, float cutCons, const float *MDK_W);
  
  /* Attribute memory deletion */
  void freeAlignment(void);

 public:
  
  /* Constructors */
  
  /** \brief Null constructor.
   *
   * This construction method initializates all attributes 
   * of the new object with 0 or NULL value.
   */
  alignment(void);
  
  /* Copy constructor */
  /** \brief Assignment constructor.
   * \param _alignmentMatrix Matrix containing the new alignment matrix.
   * \param _speciesNames Vector containing the species names of the new alignment.
   * \param _species Number of species of the new alignment.
   * \param _aminos Number of aminos of the new alignment.
   *
   * Constructor that copies all parameters to equivalent attributes of the object
   */
  alignment(char **_alignmentMatrix, char **_speciesNames, int _species, int _aminos);   
  
  /* Destructor */
  /** \brief Destructor.
   *
   * Destruction method that frees previously allocated memory.
   */
  ~alignment(void);
    

  /* Basic operations  */
  
  /** \brief Alignment load method.
   * \param alignmentFile Alignment file name.
   * \return \e true if the load is ok, \e false if the load was wrong (i.e. the file doesn't exists).
   * 
   * Method that loads an alignment from a file.
   */
  bool loadAlignment(char *alignmentFile);

  /** \brief Alignment storing method.
   * \param destFile Destination file name of the alignment.
   * \return \e true if all is ok, \e false if there were errors (i.e. the file doesn't exists).
   *
   * Method that saves an alignment to a file.
   */
  bool saveAlignment(char *destFile);

  /** \brief Alignment printing method.
   * \return \e true if all is ok, \e false if there were errors.
   *
   * Method that prints an alignment to the standard output.
   */
  bool printAlignment(void);
  
  
  /* Alignment trimming. */

  /** \brief Alignment trimming using gap method.
   * \param baseLine base line, security, percentage of colums in the alignment.
   * \param threshold maximum percentage of gaps permitted per column.
   * \return the clean alignment if all is ok, \e NULL if there were errors
   *
   * Method that cleans the alignment using gap method, and generates a new,
   * clean alignment.
   */
  alignment *cleanGaps(float baseLine, float threshold);
  
  /** \brief Alignment trimming using conservation method.
   * \param baseLine base line, security, percentage of colums in the alignment.
   * \return the clean alignment if all is ok, \e NULL if there were errors
   *
   * Method that cleans the alignment using conservation method, and generates a new,
   * clean alignment.
   */
  alignment *cleanConservation(float baseLine, float conservationPct);

  /** \brief Alignment trimming using gap method and conservation method.
   * \param baseLine base line, security, percentage of colums in the alignment.
   * \param threshold maximum percentage of gaps permitted per column.
   * \param conservationPct minimum conservatoin percentage permitted in the clean alignment.
   * \return the clean alignment if all is ok, \e NULL if there were errors.
   *
   * Method that cleans the alignment using both gaps method and conservation method, and
   * generates a new, clean alignment.
   */
  alignment *clean(float baseLine, float threshold, float conservationPct);

  /** \brief Alignment trimming using a vector of comparison values.
   * \param threshold minimal comparison value permitted per column.
   * \param vectValue comparison values vector from the alignment to clean.
   * \return the clean alignment if all is ok, \e NULL if there were errors
   *
   * Method that cleans the alignment using vector of comparison values, and generates
   *  a new clean alignment.
   */
  alignment *cleanCompareFile(float threshold, float *vectValues);
  
  /** \brief Alignment automatic trimming strict method.
   * \return the clean alignment if all is ok, \e NULL if there were errors.
   *
   * Method that cleans, automaticly, the alignment using a combinated method between the first and
   * second slope of the ratio between the alignment's lenght and gaps percentage and, genenerates 
   * a new clean alignment.
   */
  alignment *cleanMixSlope(void);

  /** \brief Alignment automatic trimming relaxed method.
   * \return the clean alignment if all is ok, \e NULL if there were errors.
   *
   * Method that cleans, automaticly, the alignment using a second slope method between alignment's lenght
   * and gaps percentage ratio and, genenerates a new clean alignment.
   */
  alignment *clean2ndSlope(void);

   /* Statistics calculation */
  
  /** \brief Basic conservation statistics calculation.
   * \return \e true if all is ok, \e false if there were errors (i.e. there is no similarity matrix defined
   * in conservation statistics).
   *
   * This method calculates conservation statistics with the previously defined similarity matrix.
   */
  bool calculateConservationStats(void);

  /** \brief Conservation statistics calculation.
   * \param sm similarity matrix used for the statistics calculation.
   * \return \e true if all is ok, \e false if there were errors.
   *
   * This method calculates conservation statistics using the \b sm similarity matrix.
   */ 
  bool calculateConservationStats(similarityMatrix *sm);

  /** \brief Gap statistics calculation.
   * \return \e true if all is ok, \e false if there were errors.
   *
   * This method calculates gap statistics without window calculation (half window value = 0).
   */
  bool calculateGapStats(void);

  /** \brief Gap statistics calculation using a half-window value.
   * \param hwindow Half window used for window statistics calculation.
   * \return \e true if all is ok, \e false if there were errors
   *
   * This method calculates gap statistics using \b hwindow half window size.
   */
  bool calculateGapStats(int hwindow);


  /* Output Statistics */

  /** \brief Printing normal gap statistics method.
   *
   * This method prints gap statistics for each column.
   */
  void printStatisticsGapsColumns(void);

  /** \brief Printing accumulated gap statistics method.
   *
   * This method prints accumulated gap statistics.
   */
  void printStatisticsGapsTotal(void);

  /** \brief Printing conservation values method.
   *
   * This method prints conservation value for each column of the alignment.
   */
  void printStatisticsConservationColumns(void);

  /** \brief Printing accumulated conservation statistics method.
   * This method prints the accumulated number of columns for each conservation 
   * value from the the alignment.
   */
  void printStatisticsConservationTotal(void);


   /* Alignment's Info */

  /** \brief Gets alignment's species number.
   * \return the alignment's species number.
   *
   * This method returns the alignment's species number.
   */
  int getNumSpecies(void);

  /** \brief Gets alignment's species names.
   * \param characters' matrix used to storage species names.
   *
   * This method returns the alignment's species names.
   */
  void getSpecies(char **);

  /** \brief Gets alignment's amino acids number.
   * \return the alignment's amino acids number.
   *
   * This method returns the alignment's amino acids number.
   */
  int getNumAminos(void);


   /* Alignments' Compare */

  /** \brief Building alignment's sequence matrix method.
   *
   * This method builds an alignment's sequence matrix.
   */
  void sequenMatrix(void);

  /** \brief Printing alignment's sequence matrix method.
   *
   * This method prints an alignment's sequence matrix.
   */
  void printSequenMatrix(void);

  /** \brief Returns a column from alignment's sequence matrix.
   * \param colum, sequence matrix index 
   * \param columnSeqMatrix, vector used to storage a column from alignment sequence matrix.
   *
   * This method returns a column from alignment sequence matrix.
   */
  void getColumnSeqMatrix(int, int *);

  /** \brief Returns a column from alignment's sequence matrix.
   * \param value to look in a sequence matrix row.
   * \param sequence matrix row where look for a value.
   * \param columnSeqMatrix, vector used to storage a column from alignment sequence matrix.
   *
   * Method that returns a column from the aligment's sequence matrix with the same value that 
   * "value" at matrix's position (row, i)
   */
  void getColumnSeqMatrix(int, int, int *);

};

#endif

