/** \file statisticsConservation.h
 * \brief File containing \ref statisticsConservation class definition.
 *
 * This file contains the \ref statisticsConservation class definition.
 */

#ifndef STATISTICS_CONSERVATION_H
#define STATISTICS_CONSERVATION_H

#include "similarityMatrix.h"
#include "statisticsGaps.h"


/** \brief Conservation statistics class.
 *
 * This class defines the conservation statistics related to a multiple sequence alignment.
 */

class statisticsConservation{
 private:
  /* Conservation vectors */
  float *Q;
  float *MDK;
  float *MDK_Pond;
  float *MDK_Window;

  int freqBlocks[100];
  
  /* Number of columns and sequences of the alignment */
  int columns;
  int sequences;
  
  /* Identity weight matrix between alignment rows */
  float **matrixIdentity;
  
  /* Half window size */
  int halfWindow;
  
  /* Similarity matrix used to conservation calculations */
  similarityMatrix *simMatrix;
  
  /* Reference to a statisticsGaps class */
  statisticsGaps *sgaps;
  
  float interv;

  /* Private methods */
  void calculateMatrixIdentity(char **alignmentMatrix, int species, int aminos);
  void memoryAllocation(void);
  void memoryDeletion(void);

 public:

  /* Constructors */
  /** \brief Null constructor.
   *
   * This constructor sets null values in the class attributes.
   */
  statisticsConservation(void);

  /** \brief Constructor.
   * \param alignmentMatrix Multiple sequence alignment of the \ref alignment class associated with the object.
   * \param species Number of species of the alignment.
   * \param aminos Number of columns of the alignment.
   * \param sg Gap statistics object, used for obtain gap statistics for conservation value weighting.
   * 
   * This constructor calculates basic statistics from the alignment matrix, and stores the sg pointer
   * for later calculations.
   */
  statisticsConservation(char **alignmentMatrix, int species, int aminos, statisticsGaps *sg);
  
  /* Destructor */
  /** \brief Destructor.
   *
   * Destruction method.
   */
  ~statisticsConservation(void);
  
  /** \brief Similarity matrix loading method.
   * \param sm Pointer to the \ref similarityMatrix object.
   * \return \b true if al is ok, \b false if the pointer is \b NULL
   *
   * This method sets the similarity matrix for conservation statistics
   * calculation.
   */
  bool setSimilarityMatrix(similarityMatrix *sm);
  
  /** \brief Vectors calculation method.
   * \param alignmentMatrix Multiple sequence alignment of the \ref alignment class associated with the object.
   * \param species Number of species of the alignment.
   * \param aminos Number of columns of the alignment.
   * \return \b true if all is ok, \b false if there were errors.
   *
   * This method calculates conservation statistics vectors, using the alignment matrix given.
   */
  bool calculateVectors(char **alignmentMatrix, int species, int aminos);
  
  /** \brief Conservation cut point calculation.
   * \param baseLine % of columns in the alignment that we put as a minimum for the cleaning process.
   * \param conservationPct minimum conservation value (between 0 and 1) permitted in the clean alignment.
   * \return Conservation cut point value.
   *
   * This method calculates the conservation cut point value using the two given parameters.
   */
  float calcCutPoint(float baseLine, float conservationPct);
  
  /** \brief Applying window method.
   * \return \b true if all is ok, \b false if something goes wrong (i.e. the half window value has not changed
   * and we don't need to recalculate window vectors)
   *
   * This method calculates the average window statistics of the alignment. The window value is obtained
   * using a method of the gap statistics reference.
   */
  bool applyWindow(void);
  
  /** \brief Obtain Mdkw vector.
   * \return a reference to the Mdkw vector.
   *
   * This method returns a reference to the Mdkw vector, used later to
   * clean the alignment using conservation method.
   */
  float *getMdkwVector(void);

  /** \brief Similarity matrix definition checking.
   * \return \b true if a matrix is previously defined, \b false if not.
   *
   * This method returns if there is a previously defined similarity matrix
   * in the conservation statistics object.
   */
  bool isSimMatrixDef(void);

  /** \brief Previously vectors calculation checking.
   * \return \b true if vectors are previously calculated, \b false if not.
   *
   * This method returns if object vectors have been previously calculated.
   */
  bool vectorsCalculated(void);

  /** \brief Per-column conservation values printing.
   *
   * This method prints the conservation value of each column of the alignment.
   */
  void printConservationColumns(int aminos);

  /** \brief Per-column conservation values printing.
   *
   * This method prints the accumulated statistics related to the number of 
   * columns for a determinated conservation value.
   */
  void printConservationAcl(int aminos);    
};

#endif
