/** \file statisticsGaps.h
 * \brief \ref statisticsGaps class defininition file.
 *
 * In this file the class \ref statisticsGaps is defined.
 */


#ifndef STATISTICSGAPS_H
#define STATISTICSGAPS_H

#include <iostream.h>
#include <iomanip.h>

#include "utils.h"

#define GAPTHLDSECURITY		10
#define GAPBASELINESECURITY	60

/** \brief This class implements the gaps statistics calculation and manipulation.
 *
 * This class implements attributes and methods that provide to the \ref alignment
 * class with gap statistics calculation, storing and manipulation.
 */
class statisticsGaps{
  int *gapsInColumn;
  int *numColumnsWithGaps;
  int *aminosXInColumn;
  int *gapsWindow;

  int columns;
  int columnLength;
  int maxGaps;
  int halfWindow;

 public:
  /** \brief Null class constructor.
   *
   * This constructor sets attributes to null values.
   */
  statisticsGaps();

  /** \brief Alignment constructor.
   * \param alignmentMatrix The multiple sequence alignment matrix.
   * \param species Number of species of the alignment matrix.
   * \param aminos Number of columns of the alignment matrix.
   * 
   * This constructor creates a new statisticsGaps object from an alignmentMatrix,
   * calculating some important statistics that are used to calculate cut points
   * for alignment cleaning.
   */
  statisticsGaps(char **alignmentMatrix, int species, int aminos);

  /** \brief Class destructor.
   *
   * Class destructor.
   */
  ~statisticsGaps();
  
  /** \brief Average window statistics calculation method.
   * \param hwin Half-window used to calculate window statistics.
   * \return \b true if all is ok, \b false if the calculation was wrong (i.e. the half-window value is the same of a previous calculation).
   *
   * This method implements the average window calculation of the number of gaps of the columns.
   */
  bool applyWindow(int hwin);

  /** \brief Cut point calculation.
   * \param gapBaseLine Percentage of columns in the alignment that at least we will not eliminate (security minimum)
   * \param gapThreshold Maximum percentage of gaps in a column permitted in the clean alignment.
   * \return The cut point value.
   *
   * This method calculates a cut point using the two given parameters.
   */
  int calcCutPoint(float gapBaseLine, float gapThreshold);

  /** \brief Automatic cut point strict calculation.
   * \return The cut point value.
   * 
   * This method calculates the cut point for the alignment, using the gap statistics and
   * the ratio between the first and second slope.
   */
  int calcCutPointMixSlope(void);

  /** \brief Automatic cut point relaxed calculation.
   * \return The cut point value.
   * 
   * This method calculates the cut point for the alignment, using the gap
   * statistics and the second slope ratio.
   */
  int calcCutPoint2ndSlope(void);

  /* Useful methods that the statisticsConservation and alignment clases will be use */
  
  /** \brief Obtain the gaps window vector.
   * \return A reference to the object gaps window vector.
   *
   * This method returs a reference to the gaps window statistics vector.
   */
  int *getGapsWindow();

  /** \brief Obtain the indeterminations vector.
   * \return A reference to the object indeterminations vector.
   *
   * This method returs a reference to the indeterminatios statistics vector.
   */
  int *getIndVector();

  /** \brief Obtain the half window value.
   * \return The half window value of the object statistics.
   *
   * This method returs the half window value of the statistics object.
   */
  int getHalfWindow();

  /** \brief Per-column statistics printing.
   *
   * This method prints gap statistics for each column of the alignment.
   */
  void printGapsColumns();

  /** \brief Accumulated statistics printing.
   *
   * This method prints columns per gaps accumulated statistics.
   */
  void printGapsAcl();
};

#endif
